@@ -8,11 +8,15 @@
 # include <config.h>
 #endif
 
+#ifdef USE_SNPRINTB
+# include <util.h>
+#endif
 #include "ntpd.h"
 #include "ntp_io.h"
 #include "ntp_unixtime.h"
 #include "ntp_stdlib.h"
 
+#include <limits.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -39,8 +43,6 @@
 #define CLOCK_FLL	.25	/* FLL loop gain */
 #define	CLOCK_FLOOR	.0005	/* startup offset floor (s) */
 #define	CLOCK_ALLAN	11	/* Allan intercept (log2 s) */
-#define CLOCK_DAY	86400.	/* one day in seconds (s) */
-#define CLOCK_JUNE	(CLOCK_DAY * 30) /* June in seconds (s) */
 #define CLOCK_LIMIT	30	/* poll-adjust threshold */
 #define CLOCK_PGATE	4.	/* poll-adjust gate */
 #define PPS_MAXAGE	120	/* kernel pps signal timeout (s) */
@@ -125,6 +127,11 @@ static int loop_started;	/* TRUE after LOOP_DRIFTINIT */
 static void rstclock (int, double); /* transition function */
 static double direct_freq(double); /* direct set frequency */
 static void set_freq(double);	/* set frequency */
+#ifndef PATH_MAX
+# define PATH_MAX MAX_PATH
+#endif
+static char relative_path[PATH_MAX + 1]; /* relative path per recursive make */
+static char *this_file = NULL;
 
 #ifdef KERNEL_PLL
 static struct timex ntv;	/* ntp_adjtime() parameters */
@@ -142,7 +149,7 @@ static	void	stop_kern_loop(void);
 int	ntp_enable = TRUE;	/* clock discipline enabled */
 int	pll_control;		/* kernel support available */
 int	kern_enable = TRUE;	/* kernel support enabled */
-int	pps_enable;		/* kernel PPS discipline enabled */
+int	hardpps_enable;		/* kernel PPS discipline enabled */
 int	ext_enable;		/* external clock enabled */
 int	pps_stratum;		/* pps stratum */
 int	allow_panic = FALSE;	/* allow panic correction */
@@ -178,6 +185,35 @@ static sigjmp_buf env;		/* environment var. for pll_trap() */
 #endif /* SIGSYS */
 #endif /* KERNEL_PLL */
 
+static void
+sync_status(const char *what, int ostatus, int nstatus)
+{
+	char obuf[256], nbuf[256], tbuf[1024];
+#if defined(USE_SNPRINTB) && defined (STA_FMT)
+	snprintb(obuf, sizeof(obuf), STA_FMT, ostatus);
+	snprintb(nbuf, sizeof(nbuf), STA_FMT, nstatus);
+#else
+	snprintf(obuf, sizeof(obuf), "%04x", ostatus);
+	snprintf(nbuf, sizeof(nbuf), "%04x", nstatus);
+#endif
+	snprintf(tbuf, sizeof(tbuf), "%s status: %s -> %s", what, obuf, nbuf);
+	report_event(EVNT_KERN, NULL, tbuf);
+}
+
+/*
+ * file_name - return pointer to non-relative portion of this C file pathname
+ */
+static char *file_name(void)
+{
+	if (this_file == NULL) {
+	    (void)strncpy(relative_path, __FILE__, PATH_MAX);
+	    for (this_file=relative_path;
+		*this_file && ! isalnum((unsigned char)*this_file);
+		this_file++) ;
+	}
+	return this_file;
+}
+
 /*
  * init_loopfilter - initialize loop filter data
  */
@@ -192,6 +228,111 @@ init_loopfilter(void)
 	freq_cnt = (int)clock_minstep;
 }
 
+#ifdef KERNEL_PLL
+/*
+ * ntp_adjtime_error_handler - process errors from ntp_adjtime
+ */
+static void
+ntp_adjtime_error_handler(
+	const char *caller,	/* name of calling function */
+	struct timex *ptimex,	/* pointer to struct timex */
+	int ret,		/* return value from ntp_adjtime */
+	int saved_errno,	/* value of errno when ntp_adjtime returned */
+	int pps_call,		/* ntp_adjtime call was PPS-related */
+	int tai_call,		/* ntp_adjtime call was TAI-related */
+	int line		/* line number of ntp_adjtime call */
+	)
+{
+	switch (ret) {
+	    case -1:
+		switch (saved_errno) {
+		    case EFAULT:
+			msyslog(LOG_ERR, "%s: %s line %d: invalid struct timex pointer: 0x%lx",
+			    caller, file_name(), line,
+			    (long)((void *)ptimex)
+			);
+		    break;
+		    case EINVAL:
+			msyslog(LOG_ERR, "%s: %s line %d: invalid struct timex \"constant\" element value: %ld",
+			    caller, file_name(), line,
+			    (long)(ptimex->constant)
+			);
+		    break;
+		    case EPERM:
+			if (tai_call) {
+			    errno = saved_errno;
+			    msyslog(LOG_ERR,
+				"%s: ntp_adjtime(TAI) failed: %m",
+				caller);
+			}
+			errno = saved_errno;
+			msyslog(LOG_ERR, "%s: %s line %d: ntp_adjtime: %m",
+			    caller, file_name(), line
+			);
+		    break;
+		    default:
+			msyslog(LOG_NOTICE, "%s: %s line %d: unhandled errno value %d after failed ntp_adjtime call",
+			    caller, file_name(), line,
+			    saved_errno
+			);
+		    break;
+		}
+	    break;
+#ifdef TIME_OK
+	    case TIME_OK: /* 0 no leap second warning */
+		/* OK means OK */
+	    break;
+#endif
+#ifdef TIME_INS
+	    case TIME_INS: /* 1 positive leap second warning */
+		msyslog(LOG_INFO, "%s: %s line %d: kernel reports positive leap second warning state",
+		    caller, file_name(), line
+		);
+	    break;
+#endif
+#ifdef TIME_DEL
+	    case TIME_DEL: /* 2 negative leap second warning */
+		msyslog(LOG_INFO, "%s: %s line %d: kernel reports negative leap second warning state",
+		    caller, file_name(), line
+		);
+	    break;
+#endif
+#ifdef TIME_OOP
+	    case TIME_OOP: /* 3 leap second in progress */
+		msyslog(LOG_INFO, "%s: %s line %d: kernel reports leap second in progress",
+		    caller, file_name(), line
+		);
+	    break;
+#endif
+#ifdef TIME_WAIT
+	    case TIME_WAIT: /* 4 leap second has occured */
+		msyslog(LOG_INFO, "%s: %s line %d: kernel reports leap second has occured",
+		    caller, file_name(), line
+		);
+	    break;
+#endif
+#ifdef TIME_ERROR
+	    case TIME_ERROR: /* loss of synchronization */
+		if (pps_call && !(ptimex->status & STA_PPSSIGNAL))
+			report_event(EVNT_KERN, NULL,
+			    "PPS no signal");
+		errno = saved_errno;
+		DPRINTF(1, ("kernel loop status (%s) %d %m\n",
+			k_st_flags(ptimex->status), errno));
+	    break;
+#endif
+	    default:
+		msyslog(LOG_NOTICE, "%s: %s line %d: unhandled return value %d from ntp_adjtime in %s at line %d",
+		    caller, file_name(), line,
+		    ret,
+		    __func__, __LINE__
+		);
+	    break;
+	}
+	return;
+}
+#endif
+
 /*
  * local_clock - the NTP logical clock loop filter.
  *
@@ -212,6 +353,7 @@ local_clock(
 {
 	int	rval;		/* return code */
 	int	osys_poll;	/* old system poll */
+	int	ntp_adj_ret;	/* returned by ntp_adjtime */
 	double	mu;		/* interval since last update */
 	double	clock_frequency; /* clock frequency */
 	double	dtemp, etemp;	/* double temps */
@@ -529,6 +671,9 @@ local_clock(
 			    dtemp);
 			ntv.constant = sys_poll - 4;
 #endif /* STA_NANO */
+			if (ntv.constant < 0)
+				ntv.constant = 0;
+
 			ntv.esterror = (u_int32)(clock_jitter * 1e6);
 			ntv.maxerror = (u_int32)((sys_rootdelay / 2 +
 			    sys_rootdisp) * 1e6);
@@ -537,17 +682,18 @@ local_clock(
 			/*
 			 * Enable/disable the PPS if requested.
 			 */
-			if (pps_enable) {
+			if (hardpps_enable) {
+				ntv.status |= (STA_PPSTIME | STA_PPSFREQ);
 				if (!(pll_status & STA_PPSTIME))
-					report_event(EVNT_KERN,
-					    NULL, "PPS enabled");
-				ntv.status |= STA_PPSTIME | STA_PPSFREQ;
+					sync_status("PPS enabled",
+						pll_status,
+						ntv.status);
 			} else {
+				ntv.status &= ~(STA_PPSTIME | STA_PPSFREQ);
 				if (pll_status & STA_PPSTIME)
-					report_event(EVNT_KERN,
-					    NULL, "PPS disabled");
-				ntv.status &= ~(STA_PPSTIME |
-				    STA_PPSFREQ);
+					sync_status("PPS disabled",
+						pll_status,
+						ntv.status);
 			}
 			if (sys_leap == LEAP_ADDSECOND)
 				ntv.status |= STA_INS;
@@ -560,12 +706,8 @@ local_clock(
 		 * the pps. In any case, fetch the kernel offset,
 		 * frequency and jitter.
 		 */
-		if (ntp_adjtime(&ntv) == TIME_ERROR) {
-			if (pps_enable && !(ntv.status & STA_PPSSIGNAL))
-				report_event(EVNT_KERN, NULL,
-				    "PPS no signal");
-			DPRINTF(1, ("kernel loop status (%s) %d %m\n",
-				k_st_flags(ntv.status), errno));
+		if ((ntp_adj_ret = ntp_adjtime(&ntv)) != 0) {
+		    ntp_adjtime_error_handler(__func__, &ntv, ntp_adj_ret, errno, hardpps_enable, 0, __LINE__ - 1);
 		}
 		pll_status = ntv.status;
 #ifdef STA_NANO
@@ -594,7 +736,9 @@ local_clock(
 			loop_tai = sys_tai;
 			ntv.modes = MOD_TAI;
 			ntv.constant = sys_tai;
-			ntp_adjtime(&ntv);
+			if ((ntp_adj_ret = ntp_adjtime(&ntv)) != 0) {
+			    ntp_adjtime_error_handler(__func__, &ntv, ntp_adj_ret, errno, 0, 1, __LINE__ - 1);
+			}
 		}
 #endif /* STA_NANO */
 	}
@@ -817,6 +961,7 @@ set_freq(
 	)
 {
 	const char *	loop_desc;
+	int ntp_adj_ret;
 
 	drift_comp = freq;
 	loop_desc = "ntpd";
@@ -828,7 +973,9 @@ set_freq(
 			loop_desc = "kernel";
 			ntv.freq = DTOFREQ(drift_comp);
 		}
-		ntp_adjtime(&ntv);
+		if ((ntp_adj_ret = ntp_adjtime(&ntv)) != 0) {
+		    ntp_adjtime_error_handler(__func__, &ntv, ntp_adj_ret, errno, 0, 0, __LINE__ - 1);
+		}
 	}
 #endif /* KERNEL_PLL */
 	mprintf_event(EVNT_FSET, NULL, "%s %.3f PPM", loop_desc,
@@ -841,6 +988,7 @@ static void
 start_kern_loop(void)
 {
 	static int atexit_done;
+	int ntp_adj_ret;
 
 	pll_control = TRUE;
 	ZERO(ntv);
@@ -848,7 +996,7 @@ start_kern_loop(void)
 	ntv.status = STA_PLL;
 	ntv.maxerror = MAXDISPERSE;
 	ntv.esterror = MAXDISPERSE;
-	ntv.constant = sys_poll;
+	ntv.constant = sys_poll; /* why is it that here constant is unconditionally set to sys_poll, whereas elsewhere is is modified depending on nanosecond vs. microsecond kernel? */
 #ifdef SIGSYS
 	/*
 	 * Use sigsetjmp() to save state and then call ntp_adjtime(); if
@@ -861,8 +1009,11 @@ start_kern_loop(void)
 		msyslog(LOG_ERR, "sigaction() trap SIGSYS: %m");
 		pll_control = FALSE;
 	} else {
-		if (sigsetjmp(env, 1) == 0)
-			ntp_adjtime(&ntv);
+		if (sigsetjmp(env, 1) == 0) {
+			if ((ntp_adj_ret = ntp_adjtime(&ntv)) != 0) {
+			    ntp_adjtime_error_handler(__func__, &ntv, ntp_adj_ret, errno, 0, 0, __LINE__ - 1);
+			}
+		}
 		if (sigaction(SIGSYS, &sigsys, NULL)) {
 			msyslog(LOG_ERR,
 			    "sigaction() restore SIGSYS: %m");
@@ -870,7 +1021,9 @@ start_kern_loop(void)
 		}
 	}
 #else /* SIGSYS */
-	ntp_adjtime(&ntv);
+	if ((ntp_adj_ret = ntp_adjtime(&ntv)) != 0) {
+	    ntp_adjtime_error_handler(__func__, &ntv, ntp_adj_ret, errno, 0, 0, __LINE__ - 1);
+	}
 #endif /* SIGSYS */
 
 	/*
@@ -1010,6 +1163,21 @@ loop_config(
 		break;
 
 	case LOOP_KERN_CLEAR:
+#if 0		/* XXX: needs more review, and how can we get here? */
+#ifndef LOCKCLOCK
+# ifdef KERNEL_PLL
+		if (pll_control && kern_enable) {
+			memset((char *)&ntv, 0, sizeof(ntv));
+			ntv.modes = MOD_STATUS;
+			ntv.status = STA_UNSYNC;
+			ntp_adjtime(&ntv);
+			sync_status("kernel time sync disabled",
+				pll_status,
+				ntv.status);
+		   }
+# endif /* KERNEL_PLL */
+#endif /* LOCKCLOCK */
+#endif
 		break;
 
 	/*
@@ -1060,7 +1228,11 @@ loop_config(
 			clock_minstep = freq; 
 		break;
 
-	case LOOP_LEAP:		/* not used */
+	case LOOP_TICK:		/* tick increment (tick) */
+		set_sys_tick_precision(freq);
+		break;
+
+	case LOOP_LEAP:		/* not used, fall through */
 	default:
 		msyslog(LOG_NOTICE,
 		    "loop_config: unsupported option %d", item);
