@@ -219,9 +219,16 @@ typedef struct clocktime clocktime_t;
 /*
  * (usually) quick constant multiplications
  */
+#ifndef TIMES10
 #define TIMES10(_X_) (((_X_) << 3) + ((_X_) << 1))	/* *8 + *2 */
+#endif
+#ifndef TIMES24
 #define TIMES24(_X_) (((_X_) << 4) + ((_X_) << 3))      /* *16 + *8 */
+#endif
+#ifndef TIMES60
 #define TIMES60(_X_) ((((_X_) << 4)  - (_X_)) << 2)     /* *(16 - 1) *4 */
+#endif
+
 /*
  * generic l_abs() function
  */
@@ -587,8 +594,8 @@ cvt_rawdcf(
 			/*
 			 * invalid character (no consecutive bit sequence)
 			 */
-			dprintf(("parse: cvt_rawdcf: character check for 0x%x@%d FAILED\n",
-				 (u_int)*s, s - buffer));
+			dprintf(("parse: cvt_rawdcf: character check for 0x%x@%ld FAILED\n",
+				 (u_int)*s, (long)(s - buffer)));
 			*s = (unsigned char)~0;
 			rtc = CVT_FAIL|CVT_BADFMT;
 		}
@@ -1325,43 +1332,6 @@ check_y2k( void )
 	    break;
 	}
 
-	if ( year >= YEAR_PIVOT+1900 )
-	{
-	    /* check year % 100 code we put into dcf_to_unixtime() */
-	    ct.year = year % 100;
-	    Flag = 0;
-
-	    Observed = dcf_to_unixtime( &ct, &Flag );
-
-	    if ( Observed != Expected  ||  Flag )
-	    {   /* time difference */
-		fprintf( stdout, 
-"%04d: dcf_to_unixtime(%d,%d) FAILURE: was=%lu s/b=%lu  (%ld)\n",
-		   year, (int)ct.year, (int)Flag, 
-		   (unsigned long)Observed, (unsigned long)Expected,
-		   ((long)Observed - (long)Expected) );
-		Error(year);
-		break;
-	    }
-
-	    /* check year - 1900 code we put into dcf_to_unixtime() */
-	    ct.year = year - 1900;
-	    Flag = 0;
-
-	    Observed = dcf_to_unixtime( &ct, &Flag );
-
-	    if ( Observed != Expected  ||  Flag ) {   /* time difference */
-		    fprintf( stdout, 
-			     "%04d: dcf_to_unixtime(%d,%d) FAILURE: was=%lu s/b=%lu  (%ld)\n",
-			     year, (int)ct.year, (int)Flag, 
-			     (unsigned long)Observed, (unsigned long)Expected,
-			     ((long)Observed - (long)Expected) );
-		    Error(year);
-		break;
-	    }
-
-
-	}
     }
 
     return ( Fatals );
